@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 
-/* @version $Id: jsvc-unix.c 1130635 2011-06-02 16:29:44Z mturk $ */
+/* @version $Id: jsvc-unix.c 1198399 2011-11-06 16:20:44Z mturk $ */
 #include "jsvc.h"
 
 #include <signal.h>
@@ -45,9 +45,6 @@
 #define F_ULOCK 0               /* Unlock a previously locked region */
 #define F_LOCK  1               /* Lock a region for exclusive use */
 #endif
-#ifndef JSVC_UMASK
-#define JSVC_UMASK 0077
-#endif
 extern char **environ;
 
 static mode_t envmask;          /* mask to create the files */
@@ -180,14 +177,12 @@ static cap_value_t caps_std[] = {
     CAP_NET_BIND_SERVICE,
     CAP_SETUID,
     CAP_SETGID,
-    CAP_DAC_READ_SEARCH,
-    CAP_DAC_OVERRIDE
+    CAP_DAC_READ_SEARCH
 };
 
 static cap_value_t caps_min[] = {
     CAP_NET_BIND_SERVICE,
-    CAP_DAC_READ_SEARCH,
-    CAP_DAC_OVERRIDE
+    CAP_DAC_READ_SEARCH
 };
 
 #define CAPS     1
@@ -197,31 +192,42 @@ static int set_caps(int cap_type)
 {
     cap_t c;
     int ncap;
+    int flag = CAP_SET;
     cap_value_t *caps;
+    const char  *type;
 
     if (cap_type == CAPS) {
         ncap = sizeof(caps_std)/sizeof(cap_value_t);
         caps = caps_std;
+        type = "default";
+    }
+    else if (cap_type == CAPSMIN) {
+        ncap = sizeof(caps_min)/sizeof(cap_value_t);
+        caps = caps_min;
+        type = "min";
     }
     else {
         ncap = sizeof(caps_min)/sizeof(cap_value_t);
         caps = caps_min;
+        type = "null";
+        flag = CAP_CLEAR;
     }
     c = cap_init();
     cap_clear(c);
-    cap_set_flag(c, CAP_EFFECTIVE,   ncap, caps, CAP_SET);
-    cap_set_flag(c, CAP_INHERITABLE, ncap, caps, CAP_SET);
-    cap_set_flag(c, CAP_PERMITTED,   ncap, caps, CAP_SET);
+    cap_set_flag(c, CAP_EFFECTIVE,   ncap, caps, flag);
+    cap_set_flag(c, CAP_INHERITABLE, ncap, caps, flag);
+    cap_set_flag(c, CAP_PERMITTED,   ncap, caps, flag);
     if (cap_set_proc(c) != 0) {
-        log_error("failed setting %s capabilities.",
-                  cap_type == CAPS ? "default" : "min");
+        log_error("failed setting %s capabilities.", type);
         return -1;
     }
     cap_free(c);
     if (cap_type == CAPS)
         log_debug("increased capability set.");
-    else
+    else if (cap_type == CAPSMIN)
         log_debug("decreased capability set to min required.");
+    else
+        log_debug("dropped capabilities.");
     return 0;
 }
 
@@ -461,14 +467,15 @@ static int check_pid(arg_data *args)
 /*
  * read the pid from the pidfile
  */
-static int get_pidf(arg_data *args)
+static int get_pidf(arg_data *args, bool quiet)
 {
     int fd;
     int i;
     char buff[80];
 
     fd = open(args->pidf, O_RDONLY, 0);
-    log_debug("get_pidf: %d in %s", fd, args->pidf);
+    if (!quiet)
+        log_debug("get_pidf: %d in %s", fd, args->pidf);
     if (fd < 0) {
         /* something has gone wrong the JVM has stopped */
         return -1;
@@ -480,7 +487,8 @@ static int get_pidf(arg_data *args)
     if (i > 0) {
         buff[i] = '\0';
         i = atoi(buff);
-        log_debug("get_pidf: pid %d", i);
+        if (!quiet)
+            log_debug("get_pidf: pid %d", i);
         if (kill(i, 0) == 0)
             return i;
     }
@@ -501,7 +509,7 @@ static int check_tmp_file(arg_data *args)
     char buff[80];
     int fd;
 
-    pid = get_pidf(args);
+    pid = get_pidf(args, false);
     if (pid < 0)
         return -1;
     sprintf(buff, "/tmp/%d.jsvc_up", pid);
@@ -604,8 +612,8 @@ static int wait_child(arg_data *args, int pid)
  */
 static int stop_child(arg_data *args)
 {
-    int pid = get_pidf(args);
-    int count = 10;
+    int pid = get_pidf(args, false);
+    int count = 60;
 
     if (pid > 0) {
         /* kill the process and wait until the pidfile has been
@@ -613,8 +621,8 @@ static int stop_child(arg_data *args)
          */
         kill(pid, SIGTERM);
         while (count > 0) {
-            sleep(6);
-            pid = get_pidf(args);
+            sleep(1);
+            pid = get_pidf(args, true);
             if (pid <= 0) {
                 /* JVM has stopped */
                 return 0;
@@ -726,7 +734,7 @@ static int child(arg_data *args, home_data *data, uid_t uid, gid_t gid)
     log_debug("Waiting for a signal to be delivered");
     create_tmp_file(args);
     while (!stopping) {
-#if defined(OSD_POSIX) || defined(HAVE_KAFFEVM)
+#if defined(OSD_POSIX)
         java_sleep(60);
         /* pause(); */
 #else
@@ -1083,13 +1091,13 @@ int main(int argc, char *argv[])
     /*
      * umask() uses inverse logic; bits are CLEAR for allowed access.
      */
-    if ((~JSVC_UMASK) & 0022) {
+    if (~args->umask & 0022) {
         log_error("NOTICE: jsvc umask of %03o allows "
-                  "write permission to group and/or other", JSVC_UMASK);
+                  "write permission to group and/or other", args->umask);
     }
-    envmask = umask(JSVC_UMASK);
+    envmask = umask(args->umask);
     set_output(args->outfile, args->errfile, args->redirectstdin, args->procname);
-    log_debug("Switching umask back to %03o from %03o", envmask, JSVC_UMASK);
+    log_debug("Switching umask back to %03o from %03o", envmask, args->umask);
     res = run_controller(args, data, uid, gid);
     if (logger_pid != 0) {
         kill(logger_pid, SIGTERM);
