@@ -107,6 +107,31 @@ LPSTR __apxGetEnvironmentVariableA(APXHANDLE hPool, LPCSTR szName)
     return szRet;
 }
 
+BOOL apxAddToPathW(APXHANDLE hPool, LPCWSTR szAdd)
+{
+    LPWSTR wsAdd;
+    DWORD  rc;
+    DWORD  al;
+    
+    rc = GetEnvironmentVariableW(L"PATH", NULL, 0);
+    if (rc == 0 && GetLastError() == ERROR_ENVVAR_NOT_FOUND)
+        return FALSE;
+    al = lstrlenW(szAdd) + 6;
+    if (!(wsAdd = apxPoolAlloc(hPool, (al + rc + 1) * sizeof(WCHAR))))
+        return FALSE;
+    lstrcpyW(wsAdd, L"PATH=");        
+    lstrcatW(wsAdd, szAdd);        
+    lstrcatW(wsAdd, L";");        
+    if (!GetEnvironmentVariableW(L"PATH", wsAdd + al, rc - al)) {
+        apxLogWrite(APXLOG_MARK_SYSERR);
+        apxFree(wsAdd);
+        return FALSE;
+    }
+    SetEnvironmentVariableW(L"PATH", wsAdd + 5);
+    _wputenv(wsAdd);
+    apxFree(wsAdd);
+    return TRUE;
+}
 
 LPWSTR AsciiToWide(LPCSTR s, LPWSTR ws)
 {
@@ -122,40 +147,26 @@ LPWSTR AsciiToWide(LPCSTR s, LPWSTR ws)
     return pszSave;
 }
 
-LPSTR WideToAscii(LPCWSTR ws, LPSTR s)
-{
-    LPSTR pszSave = s;
-
-    if (!ws) {
-        *s = '\0';
-        return pszSave;
-    }
-    do {
-        *s++ = (CHAR)*ws;
-    } while( *ws++);
-    return pszSave;
-}
-
-LPSTR WideToUTF8(LPCWSTR ws)
+LPSTR WideToANSI(LPCWSTR ws)
 {
 
     LPSTR s;
-    int cch = WideCharToMultiByte(CP_UTF8, 0, ws, -1, NULL, 0, NULL, NULL);
+    int cch = WideCharToMultiByte(CP_ACP, 0, ws, -1, NULL, 0, NULL, NULL);
     s = (LPSTR)apxAlloc(cch);
-    if (!WideCharToMultiByte(CP_UTF8, 0, ws, -1, s, cch, NULL, NULL)) {
+    if (!WideCharToMultiByte(CP_ACP, 0, ws, -1, s, cch, NULL, NULL)) {
         apxFree(s);
         return NULL;
     }
     return s;
 }
 
-LPWSTR UTF8ToWide(LPCSTR cs)
+LPWSTR ANSIToWide(LPCSTR cs)
 {
 
     LPWSTR s;
-    int cch = MultiByteToWideChar(CP_UTF8, 0, cs, -1, NULL, 0);
+    int cch = MultiByteToWideChar(CP_ACP, 0, cs, -1, NULL, 0);
     s = (LPWSTR)apxAlloc(cch * sizeof(WCHAR));
-    if (!MultiByteToWideChar(CP_UTF8, 0, cs, -1, s, cch)) {
+    if (!MultiByteToWideChar(CP_ACP, 0, cs, -1, s, cch)) {
         apxFree(s);
         return NULL;
     }
@@ -178,7 +189,7 @@ LPSTR MzWideToAscii(LPCWSTR ws, LPSTR s)
     return pszSave;
 }
 
-LPSTR MzWideToUTF8(LPCWSTR ws)
+LPSTR MzWideToANSI(LPCWSTR ws)
 {
     LPSTR str;
     LPSTR s;
@@ -186,7 +197,7 @@ LPSTR MzWideToUTF8(LPCWSTR ws)
     int cch = 0;
 
     for ( ; p && *p; p++) {
-        int len = WideCharToMultiByte(CP_UTF8, 0, p, -1, NULL, 0, NULL, NULL);
+        int len = WideCharToMultiByte(CP_ACP, 0, p, -1, NULL, 0, NULL, NULL);
         if (len > 0)
             cch += len;
         while (*p)
@@ -197,7 +208,7 @@ LPSTR MzWideToUTF8(LPCWSTR ws)
 
     p = ws;
     for ( ; p && *p; p++) {
-        int len = WideCharToMultiByte(CP_UTF8, 0, p, -1, s, cch, NULL, NULL);
+        int len = WideCharToMultiByte(CP_ACP, 0, p, -1, s, cch, NULL, NULL);
         if (len > 0) {
             s = s + len;
             cch -= len;
@@ -256,7 +267,7 @@ LPWSTR apxMultiSzCombine(APXHANDLE hPool, LPCWSTR lpStrA, LPCWSTR lpStrB,
     if (lb) {
         AplMoveMemory(&rv[la], lpStrB, lb * sizeof(WCHAR));
     }
-    if (*lpdwLength)
+    if (lpdwLength)
         *lpdwLength = (la + lb + 1) * sizeof(WCHAR);
     return rv;
 }
@@ -416,6 +427,8 @@ LPTSTR apxStrCharRemove(LPTSTR szString, TCHAR chSkip)
 {
   LPTSTR p = szString;
   LPTSTR q = szString;
+  if (IS_EMPTY_STRING(szString))
+    return szString;
   while (*p) {
     if(*p != chSkip)
       *q++ = *p;
@@ -431,6 +444,8 @@ DWORD apxStrCharRemoveA(LPSTR szString, CHAR chSkip)
   LPSTR p = szString;
   LPSTR q = szString;
   DWORD c = 0;
+  if (IS_EMPTY_STRING(szString))
+    return c;
   while (*p) {
     if(*p != chSkip)
       *q++ = *p;
@@ -448,6 +463,8 @@ DWORD apxStrCharRemoveW(LPWSTR szString, WCHAR chSkip)
   LPWSTR p = szString;
   LPWSTR q = szString;
   DWORD  c = 0;
+  if (IS_EMPTY_STRING(szString))
+    return c;
   while (*p) {
     if(*p != chSkip)
       *q++ = *p;
@@ -465,6 +482,9 @@ apxStrCharReplaceA(LPSTR szString, CHAR chReplace, CHAR chReplaceWith)
 {
   LPSTR p = szString;
   LPSTR q = szString;
+  
+  if (IS_EMPTY_STRING(szString))
+    return;
   while (*p) {
     if(*p == chReplace)
       *q++ = chReplaceWith;
@@ -480,6 +500,8 @@ apxStrCharReplaceW(LPWSTR szString, WCHAR chReplace, WCHAR chReplaceWith)
 {
   LPWSTR p = szString;
   LPWSTR q = szString;
+  if (IS_EMPTY_STRING(szString))
+    return;
   while (*p) {
     if(*p == chReplace)
       *q++ = chReplaceWith;
